@@ -1,11 +1,12 @@
 use std::io::Error;
 use std::mem::size_of;
 use std::fmt::Debug;
-
 use fluvio_compression::CompressionError;
+use fluvio_types::PartitionId;
 use tracing::trace;
 
 use fluvio_compression::Compression;
+use fluvio_types::Timestamp;
 
 use crate::core::bytes::Buf;
 use crate::core::bytes::BufMut;
@@ -16,9 +17,11 @@ use crate::core::Version;
 
 use crate::Offset;
 use crate::Size;
+use crate::record::ConsumerRecord;
 use crate::record::Record;
 
 pub const COMPRESSION_CODEC_MASK: i16 = 0x07;
+pub const NO_TIMESTAMP: i64 = -1;
 
 pub trait BatchRecords: Default + Debug + Encoder + Decoder + Send + Sync {
     /// how many bytes does record wants to process
@@ -173,7 +176,11 @@ impl TryFrom<Batch> for Batch<RawRecords> {
     fn try_from(f: Batch) -> Result<Self, Self::Error> {
         let mut buf = Vec::new();
         f.records.encode(&mut buf, 0)?;
-        let records = RawRecords(buf);
+
+        let compression = f.get_compression()?;
+        let compressed_records = compression.compress(&buf)?;
+        let records = RawRecords(compressed_records);
+
         Ok(Batch {
             base_offset: f.base_offset,
             batch_len: f.batch_len,
@@ -182,6 +189,7 @@ impl TryFrom<Batch> for Batch<RawRecords> {
         })
     }
 }
+
 impl<R> Batch<R>
 where
     R: Encoder,
@@ -221,6 +229,24 @@ impl Batch {
         }
         self.header.last_offset_delta = self.records().len() as i32 - 1;
     }
+
+    pub fn into_consumer_records_iter(
+        self,
+        partition: PartitionId,
+    ) -> impl Iterator<Item = ConsumerRecord> {
+        let base_offset = self.base_offset;
+        let first_timestamp = self.header.first_timestamp;
+
+        self.records
+            .into_iter()
+            .enumerate()
+            .map(move |(relative, record)| ConsumerRecord {
+                partition,
+                offset: base_offset + relative as Offset,
+                timestamp_base: first_timestamp,
+                record,
+            })
+    }
 }
 impl Batch<RawRecords> {
     pub fn memory_records(&self) -> Result<MemoryRecords, CompressionError> {
@@ -339,8 +365,8 @@ pub struct BatchHeader {
     ///
     /// Adding this to the base_offset will give the offset of the last record in this batch
     pub last_offset_delta: i32,
-    pub first_timestamp: i64,
-    pub max_time_stamp: i64,
+    pub first_timestamp: Timestamp,
+    pub max_time_stamp: Timestamp,
     pub producer_id: i64,
     pub producer_epoch: i16,
     pub first_sequence: i32,
@@ -356,6 +382,16 @@ impl BatchHeader {
         let compression_bits = compression as i16 & COMPRESSION_CODEC_MASK;
         self.attributes = (self.attributes & !COMPRESSION_CODEC_MASK) | compression_bits;
     }
+
+    #[cfg(feature = "memory_batch")]
+    fn set_first_timestamp(&mut self, timestamp: Timestamp) {
+        self.first_timestamp = timestamp;
+    }
+
+    #[cfg(feature = "memory_batch")]
+    fn set_max_time_stamp(&mut self, timestamp: Timestamp) {
+        self.max_time_stamp = timestamp;
+    }
 }
 impl Default for BatchHeader {
     fn default() -> Self {
@@ -365,8 +401,8 @@ impl Default for BatchHeader {
             crc: 0,
             attributes: 0,
             last_offset_delta: -1,
-            first_timestamp: 0,
-            max_time_stamp: 0,
+            first_timestamp: NO_TIMESTAMP,
+            max_time_stamp: NO_TIMESTAMP,
             producer_id: -1,
             producer_epoch: -1,
             first_sequence: -1,
@@ -385,6 +421,112 @@ pub const BATCH_HEADER_SIZE: usize = size_of::<i32>()     // partition leader ep
         + size_of::<i16>()      // produce_epoch
         + size_of::<i32>(); // first sequence
 
+#[cfg(feature = "memory_batch")]
+pub mod memory {
+    use super::*;
+    use chrono::Utc;
+    pub struct MemoryBatch {
+        compression: Compression,
+        write_limit: usize,
+        current_size_uncompressed: usize,
+        is_full: bool,
+        create_time: Timestamp,
+        records: Vec<Record>,
+    }
+    impl MemoryBatch {
+        pub fn new(write_limit: usize, compression: Compression) -> Self {
+            let now = Utc::now().timestamp_millis();
+            Self {
+                compression,
+                is_full: false,
+                write_limit,
+                create_time: now,
+                current_size_uncompressed: Vec::<RawRecords>::default().write_size(0),
+                records: vec![],
+            }
+        }
+
+        pub(crate) fn compression(&self) -> Compression {
+            self.compression
+        }
+
+        /// Add a record to the batch.
+        /// Return relative offset.
+        pub fn push_record(&mut self, mut record: Record) -> Option<Offset> {
+            let relative_offset = self.records.len() as i64;
+            record.preamble.set_offset_delta(relative_offset as Offset);
+
+            let timestamp_delta = self.elapsed();
+            record.preamble.set_timestamp_delta(timestamp_delta);
+
+            let record_size = record.write_size(0);
+
+            if self.estimated_size() + record_size > self.write_limit {
+                self.is_full = true;
+                return None;
+            }
+
+            if self.estimated_size() + record_size == self.write_limit {
+                self.is_full = true;
+            }
+
+            self.current_size_uncompressed += record_size;
+
+            self.records.push(record);
+
+            Some(relative_offset)
+        }
+
+        pub fn is_full(&self) -> bool {
+            self.is_full || self.write_limit <= self.estimated_size()
+        }
+
+        pub fn elapsed(&self) -> Timestamp {
+            let now = Utc::now().timestamp_millis();
+
+            std::cmp::max(0, now - self.create_time)
+        }
+
+        fn estimated_size(&self) -> usize {
+            (self.current_size_uncompressed as f32
+                * match self.compression {
+                    Compression::None => 1.0,
+                    Compression::Gzip | Compression::Snappy | Compression::Lz4 => 0.5,
+                }) as usize
+                + Batch::<RawRecords>::default().write_size(0)
+        }
+    }
+
+    impl From<MemoryBatch> for Batch<MemoryRecords> {
+        fn from(p_batch: MemoryBatch) -> Self {
+            let mut batch = Self::default();
+            let compression = p_batch.compression();
+            let records = p_batch.records;
+
+            let len = records.len() as i32;
+
+            let header = batch.get_mut_header();
+            header.last_offset_delta = if len > 0 { len - 1 } else { len };
+
+            let first_timestamp = p_batch.create_time;
+
+            let max_time_stamp = records
+                .last()
+                .map(|r| first_timestamp + r.timestamp_delta())
+                .unwrap_or(0);
+
+            header.set_first_timestamp(first_timestamp);
+            header.set_max_time_stamp(max_time_stamp);
+
+            header.set_compression(compression);
+
+            *batch.mut_records() = records;
+
+            batch
+        }
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -617,4 +759,83 @@ mod test {
 
         assert_eq!(batch_created.last_offset_delta(), 2);
     }
+
+    #[test]
+    fn test_into_consumer_records_iter() {
+        let mut batch = Batch::from(vec![
+            Record::default(),
+            Record::default(),
+            Record::default(),
+        ]);
+
+        batch.header.first_timestamp = 1_500_000_000;
+        let partition_id = 1;
+
+        let consumer_records = batch
+            .into_consumer_records_iter(partition_id)
+            .collect::<Vec<ConsumerRecord>>();
+        assert_eq!(consumer_records.len(), 3);
+        assert_eq!(consumer_records[0].offset(), 0);
+        assert_eq!(consumer_records[1].offset(), 1);
+        assert_eq!(consumer_records[2].offset(), 2);
+
+        consumer_records.iter().for_each(|record| {
+            assert_eq!(record.timestamp(), 1_500_000_000);
+            assert_eq!(record.partition, partition_id);
+        });
+    }
+
+    #[cfg(feature = "memory_batch")]
+    #[test]
+    fn test_memory_batch() {
+        use super::memory::MemoryBatch;
+
+        let record = Record::from(("key", "value"));
+        let size = record.write_size(0);
+
+        let mut mb = MemoryBatch::new(
+            size * 4
+                + Batch::<RawRecords>::default().write_size(0)
+                + Vec::<RawRecords>::default().write_size(0),
+            Compression::None,
+        );
+
+        assert!(mb.push_record(record).is_some());
+        std::thread::sleep(std::time::Duration::from_millis(100));
+        let record = Record::from(("key", "value"));
+        assert!(mb.push_record(record).is_some());
+        std::thread::sleep(std::time::Duration::from_millis(100));
+        let record = Record::from(("key", "value"));
+        assert!(mb.push_record(record).is_some());
+
+        let batch: Batch<MemoryRecords> = mb.try_into().expect("failed to convert");
+        assert!(
+            batch.header.first_timestamp > 0,
+            "first_timestamp is {}",
+            batch.header.first_timestamp
+        );
+        assert!(
+            batch.header.first_timestamp < batch.header.max_time_stamp,
+            "first_timestamp: {}, max_time_stamp: {}",
+            batch.header.first_timestamp,
+            batch.header.max_time_stamp
+        );
+
+        let records_delta: Vec<_> = batch
+            .records()
+            .iter()
+            .map(|record| record.timestamp_delta())
+            .collect();
+        assert_eq!(records_delta[0], 0);
+        assert!(
+            (100..150).contains(&records_delta[1]),
+            "records_delta[1]: {}",
+            records_delta[1]
+        );
+        assert!(
+            (200..250).contains(&records_delta[2]),
+            "records_delta[2]: {}",
+            records_delta[2]
+        );
+    }
 }
