@@ -14,6 +14,7 @@ use bytes::BufMut;
 
 use crate::batch::BatchRecords;
 use crate::batch::MemoryRecords;
+use crate::batch::NO_TIMESTAMP;
 use crate::batch::RawRecords;
 use crate::core::{Encoder, Decoder};
 use crate::core::DecoderVarInt;
@@ -22,7 +23,9 @@ use crate::core::Version;
 
 use crate::batch::Batch;
 use crate::Offset;
+
 use fluvio_compression::CompressionError;
+use fluvio_types::Timestamp;
 
 /// maximum text to display
 static MAX_STRING_DISPLAY: Lazy<usize> = Lazy::new(|| {
@@ -341,11 +344,11 @@ impl<R: BatchRecords> Encoder for RecordSet<R> {
     }
 }
 
-#[derive(Decoder, Encoder, Default, Debug, Clone)]
+#[derive(Decoder, Default, Encoder, Debug, Clone)]
 pub struct RecordHeader {
     attributes: i8,
     #[varint]
-    timestamp_delta: i64,
+    timestamp_delta: Timestamp,
     #[varint]
     offset_delta: Offset,
 }
@@ -358,6 +361,11 @@ impl RecordHeader {
     pub fn offset_delta(&self) -> Offset {
         self.offset_delta
     }
+
+    #[cfg(feature = "memory_batch")]
+    pub(crate) fn set_timestamp_delta(&mut self, delta: Timestamp) {
+        self.timestamp_delta = delta;
+    }
 }
 
 #[derive(Default, Clone)]
@@ -422,6 +430,10 @@ impl Record {
             ..Default::default()
         }
     }
+
+    pub(crate) fn timestamp_delta(&self) -> Timestamp {
+        self.preamble.timestamp_delta
+    }
 }
 
 impl<K, V> From<(K, V)> for Record
@@ -515,6 +527,8 @@ pub struct ConsumerRecord<B = DefaultRecord> {
     pub partition: i32,
     /// The Record contents
     pub record: B,
+    /// Timestamp base of batch in which the records is present
+    pub(crate) timestamp_base: Timestamp,
 }
 
 impl<B> ConsumerRecord<B> {
@@ -549,6 +563,14 @@ impl ConsumerRecord<DefaultRecord> {
     pub fn value(&self) -> &[u8] {
         self.record.value().as_ref()
     }
+    /// Return the timestamp of the Record
+    pub fn timestamp(&self) -> Timestamp {
+        if self.timestamp_base <= 0 {
+            NO_TIMESTAMP
+        } else {
+            self.timestamp_base + self.record.timestamp_delta()
+        }
+    }
 }
 
 impl AsRef<[u8]> for ConsumerRecord<DefaultRecord> {
@@ -689,6 +711,46 @@ mod test {
         );
         assert!(decoded.key.is_none());
     }
+
+    #[test]
+    fn test_consumer_record_no_timestamp() {
+        let record = ConsumerRecord::<Record<RecordData>> {
+            timestamp_base: NO_TIMESTAMP,
+            offset: 0,
+            partition: 0,
+            record: Default::default(),
+        };
+
+        assert_eq!(record.timestamp(), NO_TIMESTAMP);
+        let record = ConsumerRecord::<Record<RecordData>> {
+            timestamp_base: 0,
+            offset: 0,
+            partition: 0,
+            record: Default::default(),
+        };
+        assert_eq!(record.timestamp(), NO_TIMESTAMP);
+    }
+
+    #[test]
+    fn test_consumer_record_timestamp() {
+        let record = ConsumerRecord::<Record<RecordData>> {
+            timestamp_base: 1_000_000_000,
+            offset: 0,
+            partition: 0,
+            record: Default::default(),
+        };
+
+        assert_eq!(record.timestamp(), 1_000_000_000);
+        let mut memory_record = Record::<RecordData>::default();
+        memory_record.preamble.timestamp_delta = 800;
+        let record = ConsumerRecord::<Record<RecordData>> {
+            timestamp_base: 1_000_000_000,
+            record: memory_record,
+            offset: 0,
+            partition: 0,
+        };
+        assert_eq!(record.timestamp(), 1_000_000_800);
+    }
 }
 
 #[cfg(feature = "file")]
